package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdCustomerContacts;
import org.jeecg.modules.basedata.service.IFdCustomerContactsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 客户联系人
 * @author： jeecg-boot
 * @date：   2019-05-08
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdCustomerContacts")
@Slf4j
public class FdCustomerContactsController {
	@Autowired
	private IFdCustomerContactsService fdCustomerContactsService;
	
	/**
	  * 分页列表查询
	 * @param fdCustomerContacts
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdCustomerContacts>> queryPageList(FdCustomerContacts fdCustomerContacts,
                                                           @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                                           @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
                                                           HttpServletRequest req) {
		Result<IPage<FdCustomerContacts>> result = new Result<IPage<FdCustomerContacts>>();
		fdCustomerContacts.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
		QueryWrapper<FdCustomerContacts> queryWrapper = QueryGenerator.initQueryWrapper(fdCustomerContacts, req.getParameterMap());
		Page<FdCustomerContacts> page = new Page<FdCustomerContacts>(pageNo, pageSize);
		queryWrapper.lambda().orderByAsc(FdCustomerContacts::getSort);
		IPage<FdCustomerContacts> pageList = fdCustomerContactsService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdCustomerContacts
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<FdCustomerContacts> add(@RequestBody @Valid FdCustomerContacts fdCustomerContacts, BindingResult bindingResult) {
		Result<FdCustomerContacts> result = new Result<FdCustomerContacts>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}

			//如果新增的联系人是默认的联系人
			if(fdCustomerContacts.getIsDefault().equals("1")){
				//查询是否有默认联系人
				LambdaQueryWrapper<FdCustomerContacts> queryWrapper=new LambdaQueryWrapper<>();
				queryWrapper.eq(FdCustomerContacts::getDelFlag,"0");
				queryWrapper.eq(FdCustomerContacts::getIsDefault,"1");
				queryWrapper.eq(FdCustomerContacts::getPkCustomerId,fdCustomerContacts.getPkCustomerId());
				List<FdCustomerContacts> fdCustomerContactsList=fdCustomerContactsService.list(queryWrapper);
				if(!fdCustomerContactsList.isEmpty()&&fdCustomerContactsList.size()>0){
					result.error500("已存在默认联系人");
					return result;
				}
			}

			fdCustomerContacts.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			fdCustomerContactsService.save(fdCustomerContacts);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:"+e.getMessage());

		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdCustomerContacts
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	public Result<FdCustomerContacts> edit(@RequestBody @Valid FdCustomerContacts fdCustomerContacts, BindingResult bindingResult) {
		Result<FdCustomerContacts> result = new Result<FdCustomerContacts>();
		FdCustomerContacts fdCustomerContactsEntity = fdCustomerContactsService.getById(fdCustomerContacts.getId());
		if(fdCustomerContactsEntity==null) {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			result.error500("未找到对应实体");
		}else {

			//修改的是否为默认联系人
			if(fdCustomerContacts.getIsDefault().equals("1")){
				//查询是否有默认联系人
				LambdaQueryWrapper<FdCustomerContacts> queryWrapper=new LambdaQueryWrapper<>();
				queryWrapper.eq(FdCustomerContacts::getDelFlag,"0");
				queryWrapper.eq(FdCustomerContacts::getIsDefault,"1");
				queryWrapper.eq(FdCustomerContacts::getPkCustomerId,fdCustomerContacts.getPkCustomerId());
				List<FdCustomerContacts> fdCustomerContactsList=fdCustomerContactsService.list(queryWrapper);
				if(!fdCustomerContactsList.isEmpty()&&fdCustomerContactsList.size()>0){
					if(!fdCustomerContacts.getId().equals(fdCustomerContactsList.get(0).getId())){
						result.error500("已存在默认联系人");
						return result;
					}
				}
			}
			boolean ok = fdCustomerContactsService.updateById(fdCustomerContacts);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdCustomerContacts> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdCustomerContacts> result = new Result<FdCustomerContacts>();
		FdCustomerContacts fdCustomerContacts = fdCustomerContactsService.getById(id);
		if(fdCustomerContacts==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdCustomerContactsService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdCustomerContacts> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdCustomerContacts> result = new Result<FdCustomerContacts>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdCustomerContactsService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询多条数据
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryByIdList")
	public Result<List<FdCustomerContacts>> queryByIdList(@RequestParam(name="id",required=true) String id) {
		Result<List<FdCustomerContacts>> result = new Result<List<FdCustomerContacts>>();
		LambdaQueryWrapper<FdCustomerContacts> lambdaQueryWrapper=new LambdaQueryWrapper<>();
		lambdaQueryWrapper.eq(FdCustomerContacts::getPkCustomerId,id);
		List<FdCustomerContacts> fdCustomerContacts = fdCustomerContactsService.list(lambdaQueryWrapper);
		if(fdCustomerContacts == null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdCustomerContacts);
			result.setSuccess(true);
		}
		return result;
	}

	 /**
	  * 通过id查询
	  * @param id
	  * @return
	  */
	 @GetMapping(value = "/queryById")
	 public Result<FdCustomerContacts> queryById(@RequestParam(name="id",required=true) String id) {
		 Result<FdCustomerContacts> result = new Result<FdCustomerContacts>();
		 FdCustomerContacts fdCustomerContacts = fdCustomerContactsService.getById(id);
		 if(fdCustomerContacts == null) {
			 result.error500("未找到对应实体");
		 }else {
			 result.setResult(fdCustomerContacts);
			 result.setSuccess(true);
		 }
		 return result;
	 }

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdCustomerContacts> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdCustomerContacts fdCustomerContacts = JSON.parseObject(deString, FdCustomerContacts.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdCustomerContacts, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdCustomerContacts> pageList = fdCustomerContactsService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "客户联系人列表");
      mv.addObject(NormalExcelConstants.CLASS, FdCustomerContacts.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("客户联系人列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdCustomerContacts> listFdCustomerContactss = ExcelImportUtil.importExcel(file.getInputStream(), FdCustomerContacts.class, params);
              for (FdCustomerContacts fdCustomerContactsExcel : listFdCustomerContactss) {
                  fdCustomerContactsService.save(fdCustomerContactsExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdCustomerContactss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
