package org.jeecg.modules.expense.controller;

import javax.servlet.http.HttpServletRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.dto.expense.ExpenseSlipAddReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipRespDTO;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.expense.entity.ReExpenseSlip;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.expense.service.ReExpenseSlipService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
   * @Author jihaosen
   * @date 2021/3/3
   */

 @Api("费用报销单")
@RestController
@RequestMapping("/expense/reExpenseSlip")
@Slf4j
public class ReExpenseSlipController {
	@Autowired
	private ReExpenseSlipService reExpenseSlipService;

	 @ApiOperation(value = "分页查询", notes = "分页查询")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
	 })
	@GetMapping(value = "/list")
	public Result<IPage<ReExpenseSlip>> queryPageList(ReExpenseSlip reExpenseSlip,
	  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
	  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
	  HttpServletRequest req) {
		Result<IPage<ReExpenseSlip>> result = new Result<IPage<ReExpenseSlip>>();
		 reExpenseSlip.setDelFlag(null);
		QueryWrapper<ReExpenseSlip> queryWrapper = QueryGenerator.initQueryWrapper(reExpenseSlip, req.getParameterMap());
		Page<ReExpenseSlip> page = new Page<ReExpenseSlip>(pageNo, pageSize);
		 queryWrapper.eq("del_flag", "0");
		IPage<ReExpenseSlip> pageList = reExpenseSlipService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	 @ApiOperation(value = "新增", notes = "新增")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="personId", value="人员id",required=false, dataType="String"),
			 @ApiImplicitParam(name="person", value="人员",required=false, dataType="String"),
			 @ApiImplicitParam(name="date", value="日期",required=false, dataType="String"),
			 @ApiImplicitParam(name="producerId", value="制单人id",required=false, dataType="String"),
			 @ApiImplicitParam(name="producer", value="制单人",required=false, dataType="String"),
			 @ApiImplicitParam(name="reviewerId", value="审核人id",required=false, dataType="String"),
			 @ApiImplicitParam(name="reviewer", value="审核人",required=false, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目id",required=false, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目名",required=false, dataType="String"),
			 @ApiImplicitParam(name="costId", value="费用项目id",required=false, dataType="String"),
			 @ApiImplicitParam(name="costName", value="费用项目名称",required=false, dataType="String"),
	 })
	@PostMapping(value = "/add")
	public Result<ReExpenseSlip> add(@RequestBody ExpenseSlipAddReqDTO reqDTO) {
		Result<ReExpenseSlip> result = new Result<ReExpenseSlip>();
		try {
			reExpenseSlipService.add(reqDTO);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	 @ApiOperation(value = "编辑", notes = "编辑")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="personId", value="人员id",required=false, dataType="String"),
			 @ApiImplicitParam(name="person", value="人员",required=false, dataType="String"),
			 @ApiImplicitParam(name="date", value="日期",required=false, dataType="String"),
			 @ApiImplicitParam(name="producerId", value="制单人id",required=false, dataType="String"),
			 @ApiImplicitParam(name="producer", value="制单人",required=false, dataType="String"),
			 @ApiImplicitParam(name="reviewerId", value="审核人id",required=false, dataType="String"),
			 @ApiImplicitParam(name="reviewer", value="审核人",required=false, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目id",required=false, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目名",required=false, dataType="String"),
			 @ApiImplicitParam(name="costId", value="费用项目id",required=false, dataType="String"),
			 @ApiImplicitParam(name="costName", value="费用项目名称",required=false, dataType="String"),
	 })
	@PutMapping(value = "/edit")
	public Result<ReExpenseSlip> edit(@RequestBody ExpenseSlipAddReqDTO reqDTO) {
		Result<ReExpenseSlip> result = new Result<ReExpenseSlip>();
		ReExpenseSlip reExpenseSlipEntity = reExpenseSlipService.getById(reqDTO.getId());
		if(reExpenseSlipEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = reExpenseSlipService.edit(reqDTO);
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	@ApiOperation(value = "通过id删除主子表", notes = "通过id删除主子表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
	})
	@DeleteMapping(value = "/delete")
	public Result<ReExpenseSlip> delete(ExpenseSlipReqDTO  reqDTO) {
		Result<ReExpenseSlip> result = new Result<ReExpenseSlip>();
		ReExpenseSlip reExpenseSlip = reExpenseSlipService.getById(reqDTO.getId());
		if(reExpenseSlip==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = reExpenseSlipService.deleteById(reExpenseSlip);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}

	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
	})
	@GetMapping(value = "/queryById")
	public Result<ReExpenseSlip> queryById(ExpenseSlipReqDTO  reqDTO) {
		Result<ReExpenseSlip> result = new Result<ReExpenseSlip>();
		ReExpenseSlip reExpenseSlip = reExpenseSlipService.getById(reqDTO.getId());
		if(reExpenseSlip==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(reExpenseSlip);
			result.setSuccess(true);
		}
		return result;
	}

	@ApiOperation(value = "通过id查询主子表", notes = "通过id查询主子表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
	})
	@GetMapping(value = "/getListById")
	public Result<ExpenseSlipRespDTO> getListById(ExpenseSlipReqDTO  reqDTO) {
		Result<ExpenseSlipRespDTO> result = new Result<ExpenseSlipRespDTO>();
		ExpenseSlipRespDTO respDTO = reExpenseSlipService.getListById(reqDTO.getId());
			result.setResult(respDTO);
			result.setSuccess(true);
		return result;
	}
}
