package org.jeecg.modules.expense.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import org.jeecg.common.dto.expense.ExpenseSlipAddReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipProjectAddReqDTO;
import org.jeecg.common.dto.expense.ExpenseSlipProjectListRespDTO;
import org.jeecg.common.dto.expense.ExpenseSlipRespDTO;
import org.jeecg.modules.expense.entity.ReExpenseSlip;
import org.jeecg.modules.expense.entity.ReExpenseSlipProject;
import org.jeecg.modules.expense.mapper.ReExpenseSlipMapper;
import org.jeecg.modules.expense.service.ReExpenseSlipProjectService;
import org.jeecg.modules.expense.service.ReExpenseSlipService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import java.util.List;

/**
 * @Description: 费用报销单
 * @author： jeecg-boot
 * @date：   2021-03-03
 * @version： V1.0
 */
@Service
public class ReExpenseSlipServiceImpl extends ServiceImpl<ReExpenseSlipMapper, ReExpenseSlip> implements ReExpenseSlipService {
    @Autowired
    private ReExpenseSlipProjectService reExpenseSlipProjectService;

    /**
     * @desc 新增
     */
    @Override
    public void add(ExpenseSlipAddReqDTO reqDTO) {
        //保存主表
        ReExpenseSlip reExpenseSlip = new ReExpenseSlip(reqDTO);
         save(reExpenseSlip);
         //保存子表
        saveChildren(reqDTO, reExpenseSlip.getId());
    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(ExpenseSlipAddReqDTO reqDTO) {
        //修改主表
        updateById(new ReExpenseSlip(reqDTO));
        //删除子表
        deleteChildren(reqDTO.getId());
        //保存子表
        saveChildren(reqDTO, reqDTO.getId());
        return true;
    }

    /**
     * @desc 根据id删除主子表
     */
    @Override
    public boolean deleteById(ReExpenseSlip reExpenseSlip) {
        reExpenseSlip.setDelFlag("1");
        updateById(reExpenseSlip);
        //删除子表
        deleteChildren(reExpenseSlip.getId());
        return true;
    }

    /**
     * @desc 根据id查询主子表
     */
    @Override
    public ExpenseSlipRespDTO getListById(String id) {
        ReExpenseSlip slip = getById(id);
        ExpenseSlipRespDTO respDTO = new ExpenseSlipRespDTO(slip);
        //设置子表
        respDTO.setDetailList(getChildren(id));
        return respDTO;
    }



    /**
     * @desc 获取子表
     */
    private List<ExpenseSlipProjectListRespDTO> getChildren(String id) {
        QueryWrapper<ReExpenseSlipProject> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("expense_id", id);
        queryWrapper.orderByAsc("sort");
        List<ReExpenseSlipProject> list = reExpenseSlipProjectService.list(queryWrapper);
        List<ExpenseSlipProjectListRespDTO> respDTOS = Lists.newArrayList();
        for(ReExpenseSlipProject project : list){
            respDTOS.add(new ExpenseSlipProjectListRespDTO(project));
        }
        return respDTOS;
    }

    /**
     * @desc 删除子表
     */
    private void deleteChildren(String id) {
        QueryWrapper<ReExpenseSlipProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("expense_id", id);
        queryWrapper.eq("del_flag","0");
        reExpenseSlipProjectService.update(new ReExpenseSlipProject("1"), queryWrapper);
    }

    /**
     * @desc 保存子表
     */
    private void saveChildren(ExpenseSlipAddReqDTO reqDTO, String id) {
        List<ExpenseSlipProjectAddReqDTO> detailList = reqDTO.getDetailList();
        List<ReExpenseSlipProject> list = Lists.newArrayList();
        int sort = 1;
        for(ExpenseSlipProjectAddReqDTO detail : detailList){
            ReExpenseSlipProject project = new ReExpenseSlipProject(detail);
            project.setExpenseId(id);
            project.setSort(sort++);
            list.add(project);
        }
        reExpenseSlipProjectService.saveBatch(list);
    }
}
