package org.jeecg.modules.ngalain.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.SpringContextUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdPersonnel;
import org.jeecg.modules.basedata.service.IFdPersonnelService;
import org.jeecg.modules.ngalain.service.NgAlainService;
import org.jeecg.modules.system.entity.SysDepart;
import org.jeecg.modules.system.entity.SysDict;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.entity.SysUserRole;
import org.jeecg.modules.system.model.SysDictTree;
import org.jeecg.modules.system.service.ISysDepartService;
import org.jeecg.modules.system.service.ISysDictService;
import org.jeecg.modules.system.service.ISysUserRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/sys/ng-alain")
public class NgAlainController {
    @Autowired
    private NgAlainService ngAlainService;
    @Autowired
    private ISysDictService sysDictService;

    @Autowired
    private ISysUserRoleService sysUserRoleService;

    @Autowired
    private IFdPersonnelService iFdPersonnelService;

    @Autowired
    private ISysDepartService iSysDepartService;

    @RequestMapping(value = "/getAppData")
    @ResponseBody
    @ApiOperation(value = "登陆之后获取菜单/用户/app信息" ,httpMethod = "POST",notes = "登陆之后获取菜单/用户/app信息", produces = "application/json")
    public Result<JSONObject> getAppData(HttpServletRequest request,String orgId) throws Exception {
        String token=request.getHeader("X-Access-Token");
        JSONObject j = new JSONObject();
        SysUser user = (SysUser) SecurityUtils.getSubject().getPrincipal();
        JSONObject userObjcet = new JSONObject();
        userObjcet.put("id", user.getId());
        userObjcet.put("name", user.getUsername());
        userObjcet.put("avatar", SpringContextUtils.getDomain() +"/fangda/sys/common/view/"+user.getAvatar());
        userObjcet.put("email", user.getEmail());
        userObjcet.put("token", token);
        userObjcet.put("personnelId", user.getPersonnelId());
        List<SysDepart> sysDeparts=iSysDepartService.getByType("1");
        //判断组织id是否为空
        if(StringUtils.isNotBlank(orgId)&&!orgId.equals("null")&&!orgId.equals("undefined")){
            //不为空则不变
//            FdPersonnel fdPersonnel=iFdPersonnelService.getById(user.getPersonnelId());
            userObjcet.put("pkOrg", orgId);
        }else{
            //为空
            //如果关联了人员根据人员组织
            if(StringUtils.isNotBlank(user.getPersonnelId())&&!user.getPersonnelId().equals("123")){
                FdPersonnel fdPersonnel=iFdPersonnelService.getById(user.getPersonnelId());
                orgId=fdPersonnel.getPkOrg();
                userObjcet.put("pkOrg", orgId);
            }else{
                //如果没有关联默认公司 admin

                orgId=sysDeparts.get(0).getPkOrg();
                userObjcet.put("pkOrg", orgId);
            }
        }

        for(SysDepart sysDepart:sysDeparts){
            if(sysDepart.getId().equals(orgId)){
                userObjcet.put("pkOrgName", sysDepart.getDepartName());
            }
        }

        userObjcet.put("realname",user.getRealname());
        //获取当前角色id
        List<SysUserRole> userRole = sysUserRoleService.list(new QueryWrapper<SysUserRole>().lambda().eq(SysUserRole::getUserId, user.getId()));
        if(!userRole.isEmpty()&&userRole.size()>0){
            userObjcet.put("roleList",userRole);
        }else{
            userObjcet.put("roleList",null);
        }

        j.put("user", userObjcet);
//        j.put("menu",ngAlainService.getMenu(user.getUsername()));
        j.put("menu",ngAlainService.getMenuByOrg(user.getUsername(),orgId));
        JSONObject app = new JSONObject();
        app.put("name", "jeecg-boot-angular");
        app.put("description", "jeecg+ng-alain整合版本");
        j.put("app", app);
        //获取用户权限标识
//        j.put("perms",ngAlainService.queryPermsByUser(user.getUsername()));
        j.put("perms",ngAlainService.queryPermsByUserAndPkorg(user.getUsername(),orgId));
        Result<JSONObject> result = new Result<JSONObject>();
        result.setSuccess(true);
        result.setResult(j);
        return result;
    }

    @RequestMapping(value = "/getDictItems/{dictCode}", method = RequestMethod.GET)
    public Object getDictItems(@PathVariable String dictCode) {
        log.info(" dictCode : "+ dictCode);
        Result<List<Map<String,Object>>> result = new Result<List<Map<String,Object>>>();
        List<Map<String,Object>> ls = null;
        try {
            ls = sysDictService.queryDictItemsByCode(dictCode);
            result.setSuccess(true);
            result.setResult(ls);
        } catch (Exception e) {
            log.info(e.getMessage());
            result.error500("操作失败");
            return result;
        }
        List<JSONObject> dictlist=new ArrayList<>();
        for (Map<String, Object> l : ls) {
            JSONObject dict=new JSONObject();
                try {
                    dict.put("value",Integer.parseInt((String)l.get("value")));
                } catch (NumberFormatException e) {
                    dict.put("value",l.get("value"));
                }
            dict.put("label",l.get("text"));
            dictlist.add(dict);
        }
        return dictlist;
    }
    @RequestMapping(value = "/getDictItemsByTable/{table}/{key}/{value}", method = RequestMethod.GET)
    public Object getDictItemsByTable(@PathVariable String table,@PathVariable String key,@PathVariable String value) {
        return this.ngAlainService.getDictByTable(table,key,value);
    }
}
