package org.jeecg.modules.prowork.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.dto.prowork.ProWorkLogicAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkLogicListReqDTO;
import org.jeecg.common.dto.prowork.ProWorkLogicRespDTO;
import org.jeecg.common.dto.prowork.ProdWorkLogicReqDTO;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.modules.prowork.entity.ProWorkLogic;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.prowork.service.ProWorkLogicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Title: Controller
 * @Description: 日志(实施，开发，服务)
 * @author： jeecg-boot
 * @date：   2021-03-02
 * @version： V1.0
 */
 @Api("日志(实施，开发，服务)接口")
@RestController
@RequestMapping("/prowork/proWorkLogic")
@Slf4j
public class ProWorkLogicController {
	@Autowired
	private ProWorkLogicService proWorkLogicService;


	 @ApiOperation(value = "分页查询接口", notes = "日志分页查询")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="type", value="1.开发 2.实施 3.服务",required=true, dataType="String"),
			 @ApiImplicitParam(name="billcode", value="单据编号",required=false, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目名称",required=false, dataType="String"),
	 })
	@GetMapping(value = "/list")
	public Result<IPage<ProWorkLogic>> queryPageList(@Valid ProWorkLogicListReqDTO reqDTO,
	 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
	 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize
	 ) {
		Result<IPage<ProWorkLogic>> result = new Result<IPage<ProWorkLogic>>();

		try {

			QueryWrapper<ProWorkLogic> queryWrapper = new QueryWrapper<>();
			Page<ProWorkLogic> page = new Page<ProWorkLogic>(pageNo, pageSize);
			if(StringUtils.isBlank(reqDTO.getType())){
				throw new JeecgBootException("参数type不能为空");
			}

			if(StringUtils.isNotBlank(reqDTO.getBillcode())) {
				queryWrapper.eq("billcode", reqDTO.getBillcode());
			}
			if(StringUtils.isNotBlank(reqDTO.getProName())) {
				queryWrapper.eq("pro_name", reqDTO.getProName());
			}
			queryWrapper.eq("type", reqDTO.getType());
			if(StringUtils.isNotBlank(reqDTO.getPkOrg())) {
				queryWrapper.eq("pk_org", reqDTO.getPkOrg());
			}

			queryWrapper.eq("del_flag", CommonConstant.STATUS_NORMAL.toString());
			queryWrapper.orderByDesc("create_time");
			IPage<ProWorkLogic> pageList = proWorkLogicService.page(page, queryWrapper);
			result.setSuccess(true);
			result.setResult(pageList);
		}catch (Exception e){
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:" + e.getMessage());
		}
		return result;
	}

	 @ApiOperation(value = "新增接口", notes = "新增日志")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="billcode", value="单据编号",required=false, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目id",required=false, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目名称",required=false, dataType="String"),
			 @ApiImplicitParam(name="startDate", value="开始日期",required=false, dataType="String"),
			 @ApiImplicitParam(name="endDate", value="结束日期",required=false, dataType="String"),
			 @ApiImplicitParam(name="reporterId", value="汇报人id",required=false, dataType="String"),
			 @ApiImplicitParam(name="reporter", value="汇报人",required=false, dataType="String"),
			 @ApiImplicitParam(name="currentUser", value="填写人(当前用户)",required=false, dataType="String"),
			 @ApiImplicitParam(name="logicId", value="日志id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proArchivesId", value="项目档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proArchivesMilestone", value="项目档案里程碑",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),
			 @ApiImplicitParam(name="duration", value="工作时长",required=true, dataType="String"),
	 })
	@PostMapping(value = "/add")
	public Result<ProWorkLogic> add(@RequestBody ProWorkLogicAddReqDTO reqDTO, BindingResult bindingResult) {
		Result<ProWorkLogic> result = new Result<ProWorkLogic>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			proWorkLogicService.add(reqDTO);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	 @ApiOperation(value = "编辑接口", notes = "编辑日志")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="billcode", value="单据编号",required=true, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目名称",required=true, dataType="String"),
			 @ApiImplicitParam(name="startDate", value="开始日期",required=true, dataType="String"),
			 @ApiImplicitParam(name="endDate", value="结束日期",required=true, dataType="String"),
			 @ApiImplicitParam(name="reporter", value="汇报人",required=true, dataType="String"),
			 @ApiImplicitParam(name="currentUser", value="填写人(当前用户)",required=true, dataType="String"),
			 @ApiImplicitParam(name="logicId", value="日志id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proArchivesId", value="项目档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proArchivesMilestone", value="项目档案里程碑",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),
			 @ApiImplicitParam(name="duration", value="工作时长",required=true, dataType="String"),
	 })
	@PutMapping(value = "/edit")
	public Result<ProWorkLogic> edit(@RequestBody ProWorkLogicAddReqDTO reqDTO) {
		Result<ProWorkLogic> result = new Result<ProWorkLogic>();
		ProWorkLogic proWorkLogicEntity = proWorkLogicService.getById(reqDTO.getId());
		if(proWorkLogicEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = proWorkLogicService.edit(reqDTO);
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}

	 @ApiOperation(value = "通过id删除接口", notes = "通过id删除日志")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
	 })
	@DeleteMapping(value = "/delete")
	public Result<ProWorkLogic> delete(ProdWorkLogicReqDTO reqDTO) {
		Result<ProWorkLogic> result = new Result<ProWorkLogic>();
		ProWorkLogic proWorkLogic = proWorkLogicService.getById(reqDTO.getId());
		if(proWorkLogic==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = proWorkLogicService.dropById(proWorkLogic);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "通过id查询接口", notes = "通过id查询日志")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
	})
	@GetMapping(value = "/queryById")
	public Result<ProWorkLogicRespDTO> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ProWorkLogicRespDTO> result = new Result<ProWorkLogicRespDTO>();
		ProWorkLogicRespDTO proWorkLogic = proWorkLogicService.queryById(id);
		if(proWorkLogic==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(proWorkLogic);
			result.setSuccess(true);
		}
		return result;
	}
}
