package org.jeecg.modules.prowork.controller;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.dto.prowork.ProWorkMilestoneAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkMilestoneReqDTO;
import org.jeecg.common.dto.prowork.ProWorkMilestoneRespDTO;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.prowork.entity.ProWorkMilestone;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.prowork.service.ProWorkMilestoneService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

 /**
   * @Author jihaosen
   * @date 2021/3/3
   */

@Api("里程碑确认单接口")
@RestController
@RequestMapping("/prowork/proWorkMilestone")
@Slf4j
 class ProWorkMilestoneController {
	@Autowired
	private ProWorkMilestoneService proWorkMilestoneService;


	 @ApiOperation(value = "分页查询接口", notes = "里程碑确认单分页查询")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="type", value="1.开发 2.实施 3.服务",required=true, dataType="String"),
	 })
	@GetMapping(value = "/list")
	public Result<IPage<ProWorkMilestone>> queryPageList(@Valid ProWorkMilestone proWorkMilestone,
		 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
		 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
		 HttpServletRequest req) {
		Result<IPage<ProWorkMilestone>> result = new Result<IPage<ProWorkMilestone>>();
		 proWorkMilestone.setDelFlag(null);
		QueryWrapper<ProWorkMilestone> queryWrapper = QueryGenerator.initQueryWrapper(proWorkMilestone, req.getParameterMap());
		 queryWrapper.eq("del_flag", "0");
		 queryWrapper.orderByDesc("create_time");
		Page<ProWorkMilestone> page = new Page<ProWorkMilestone>(pageNo, pageSize);
		IPage<ProWorkMilestone> pageList = proWorkMilestoneService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	 @ApiOperation(value = "新增", notes = "里程碑确认单新增")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="billcode", value="单据编号",required=true, dataType="String"),
			 @ApiImplicitParam(name="type", value="1.开发 2.实施 3.服务",required=true, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proCode", value="项目档案编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目档案名称",required=true, dataType="String"),
			 @ApiImplicitParam(name="cusId", value="客户档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="cusCode", value="客户编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileId", value="里程碑id",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileName", value="里程碑(计划)",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileId", value="里程碑确认(0,未确认 1，确认)",required=true, dataType="String"),
			 @ApiImplicitParam(name="confirmTime", value="确认时间",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),
			 @ApiImplicitParam(name="duration", value="工作时长",required=true, dataType="String"),
			 @ApiImplicitParam(name="updateFileId", value="文件id",required=true, dataType="String"),
			 @ApiImplicitParam(name="currentUser", value="填写人(当前用户)",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),

	 })
	@PostMapping(value = "/add")
	public Result<ProWorkMilestone> add(@RequestBody ProWorkMilestoneAddReqDTO reqDTO, BindingResult bindingResult) {
		Result<ProWorkMilestone> result = new Result<ProWorkMilestone>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			proWorkMilestoneService.add(reqDTO);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	 @ApiOperation(value = "编辑", notes = "里程碑确认单编辑")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="billcode", value="单据编号",required=true, dataType="String"),
			 @ApiImplicitParam(name="type", value="1.开发 2.实施 3.服务",required=true, dataType="String"),
			 @ApiImplicitParam(name="proId", value="项目档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="proCode", value="项目档案编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="proName", value="项目档案名称",required=true, dataType="String"),
			 @ApiImplicitParam(name="cusId", value="客户档案id",required=true, dataType="String"),
			 @ApiImplicitParam(name="cusCode", value="客户编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileId", value="里程碑id",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileName", value="里程碑(计划)",required=true, dataType="String"),
			 @ApiImplicitParam(name="mileId", value="里程碑确认(0,未确认 1，确认)",required=true, dataType="String"),
			 @ApiImplicitParam(name="confirmTime", value="确认时间",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),
			 @ApiImplicitParam(name="duration", value="工作时长",required=true, dataType="String"),
			 @ApiImplicitParam(name="updateFileId", value="文件id",required=true, dataType="String"),
			 @ApiImplicitParam(name="currentUser", value="填写人(当前用户)",required=true, dataType="String"),
			 @ApiImplicitParam(name="content", value="工作内容",required=true, dataType="String"),

	 })
	@PutMapping(value = "/edit")
	public Result<ProWorkMilestone> edit(@RequestBody ProWorkMilestoneAddReqDTO reqDTO) {
		Result<ProWorkMilestone> result = new Result<ProWorkMilestone>();
		ProWorkMilestone proWorkMilestoneEntity = proWorkMilestoneService.getById(reqDTO.getId());
		if(proWorkMilestoneEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = proWorkMilestoneService.edit(reqDTO);
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	@ApiOperation(value = "通过id删除", notes = "通过id删除主子表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
	})
	@DeleteMapping(value = "/delete")
	public Result<ProWorkMilestone> delete( ProWorkMilestoneReqDTO  reqDTO) {
		Result<ProWorkMilestone> result = new Result<ProWorkMilestone>();
		ProWorkMilestone proWorkMilestone = proWorkMilestoneService.getById(reqDTO.getId());
		if(proWorkMilestone==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = proWorkMilestoneService.dropById(proWorkMilestone);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}


	 @ApiOperation(value = "通过id查询", notes = "通过id查询主子表")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
	 })
	@GetMapping(value = "/queryById")
	public Result<ProWorkMilestoneRespDTO> queryById(ProWorkMilestoneReqDTO  reqDTO) {
		Result<ProWorkMilestoneRespDTO> result = new Result<ProWorkMilestoneRespDTO>();
		 ProWorkMilestoneRespDTO proWorkMilestone = proWorkMilestoneService.queryById(reqDTO.getId());
		if(proWorkMilestone==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(proWorkMilestone);
			result.setSuccess(true);
		}
		return result;
	}
}
