package org.jeecg.modules.report.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.dto.invoice.PurchaseDataDTO;
import org.jeecg.common.dto.payment.SlipAllListDTO;
import org.jeecg.common.dto.report.CostAccountingReqDTO;
import org.jeecg.common.dto.report.CostAccountingRespDTO;
import org.jeecg.common.dto.report.ProStatisticsRespDTO;
import org.jeecg.common.dto.report.ProStatisticsThreeRespDTO;
import org.jeecg.common.dto.report.ProStatisticsTwoRespDTO;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.modules.archives.entity.ProjectManageArchives;
import org.jeecg.modules.archives.entity.ProjectManageArchivesAndBusiness;
import org.jeecg.modules.archives.entity.ProjectManageBusinessOther;
import org.jeecg.modules.archives.service.ProjectManageArchivesAndBusinessService;
import org.jeecg.modules.archives.service.ProjectManageArchivesService;
import org.jeecg.modules.archives.service.ProjectManageBusinessOtherService;
import org.jeecg.modules.basedata.entity.BaseArchivesCollectionLine;
import org.jeecg.modules.basedata.service.IBaseArchivesCollectionLineService;
import org.jeecg.modules.invoice.entity.InvoiceManagePurchase;
import org.jeecg.modules.invoice.service.InvoiceManagePurchaseService;
import org.jeecg.modules.payment.entity.ManagerPaymentAndReceiptSlip;
import org.jeecg.modules.payment.service.ManagerPaymentAndReceiptSlipService;
import org.jeecg.modules.prowork.entity.ProWorkLogic;
import org.jeecg.modules.prowork.entity.ProWorkMilestone;
import org.jeecg.modules.prowork.service.ProWorkLogicService;
import org.jeecg.modules.prowork.service.ProWorkMilestoneService;
import org.jeecg.modules.report.entity.ChoosePlanTypeEnum;
import org.jeecg.modules.report.mapper.ProReportMapper;
import org.jeecg.modules.report.service.ProReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
   * @Author jihaosen
   * @date 2021/3/11
   */
@Service
public class ProReportServiceImpl implements ProReportService {
    @Autowired
    private ProjectManageArchivesService projectManageArchivesService;
    @Autowired
    private ProjectManageArchivesAndBusinessService projectManageArchivesAndBusinessService;
    @Autowired
    private ProReportMapper proReportMapper;
    @Autowired
    private IBaseArchivesCollectionLineService baseArchivesCollectionLineService;
    @Autowired
    private ProjectManageBusinessOtherService projectManageBusinessOtherService;
    @Autowired
    private ProWorkLogicService proWorkLogicService;
    @Autowired
    private ProWorkMilestoneService proWorkMilestoneService;
    @Autowired
    private InvoiceManagePurchaseService invoiceManagePurchaseService;
    @Autowired
    private ManagerPaymentAndReceiptSlipService managerPaymentAndReceiptSlipService;
    /**
     * @desc 获取成本核算数据
     */
    @Override
    public List<CostAccountingRespDTO> getCostAccountingData(CostAccountingReqDTO reqDTO) {
        //获取项目档案
        List<CostAccountingRespDTO> respDTOS = proReportMapper.getCostAccountingData(reqDTO);
        for(CostAccountingRespDTO respDTO : respDTOS){
           BigDecimal totalTax = respDTO.getProIncome().add(respDTO.getImIncome()).add(respDTO.getSeIncome()).add(respDTO.getOtherIncome())
                   .subtract(respDTO.getProPayment()).add(respDTO.getImPayment()).subtract(respDTO.getDePayment()).subtract(respDTO.getOtherPayment());
            respDTO.setTotalTax(totalTax);

        }
        return respDTOS;
    }

     /**
      * @desc 项目情况统计表
      */
     @Override
     public List<ProStatisticsRespDTO> getProStatistics(CostAccountingReqDTO reqDTO) {
         QueryWrapper<ProjectManageArchives> queryWrapper = new QueryWrapper();
         queryWrapper.eq("del_flag", "0");
         if(StringUtils.isNotBlank(reqDTO.getId())){
             queryWrapper.eq("id", reqDTO.getId());
         }
         if(StringUtils.isNotBlank(reqDTO.getPkOrg())){
             queryWrapper.eq("pk_org", reqDTO.getPkOrg());
         }
         List<ProjectManageArchives> archives = projectManageArchivesService.list(queryWrapper);


         List<ProStatisticsRespDTO> list = Lists.newArrayList();
         //获取树形数据
        for(ProjectManageArchives projectManageArchives : archives){
            list.add(getProStatisticsDatas(projectManageArchives,  projectManageArchives.getId()));
        }

         return list;
     }

     /**
      * @desc 获取树形数据
      */
     private ProStatisticsRespDTO getProStatisticsDatas( ProjectManageArchives archives, String id) {
         //获取项目档案里程碑信息表数据
         List<ProjectManageArchivesAndBusiness> businesses = getBusinessData(id);
         //项目档案计划信息表Map
         Map<String, List<ProjectManageArchivesAndBusiness>> businessMap = businesses.stream().collect(Collectors.groupingBy(t -> t.getPlanType()+"&"+t.getPlanId()));
         //获取收付款条线
         Map<String, BaseArchivesCollectionLine> CoLineMap = baseArchivesCollectionLineService.getDataMap();
        //获取项目档案计划信息表数据
         List<ProjectManageBusinessOther> otherList = projectManageBusinessOtherService.getAll(new ProjectManageBusinessOther(id));
         //获取里程碑确认单
         List<ProWorkMilestone> milestones = proWorkMilestoneService.getAll(new ProWorkMilestone(id));
         Map<String, List<ProWorkMilestone>> milestoneMap = milestones.stream().collect(Collectors.groupingBy(t ->t.getType()));
         //销售发票,采购发票主子表数据
         List<PurchaseDataDTO> purchaseDataDTOS = invoiceManagePurchaseService.getAll(new InvoiceManagePurchase(id));
         Map<String, List<PurchaseDataDTO>> purchaseDataMap = purchaseDataDTOS.stream().collect(Collectors.groupingBy(t -> t.getType()));
         //收付款单主子表
         List<SlipAllListDTO> slipAllListDTOS = managerPaymentAndReceiptSlipService.getAll(new ManagerPaymentAndReceiptSlip(id));
         Map<String, List<SlipAllListDTO>> slipMap = slipAllListDTOS.stream().collect(Collectors.groupingBy(t -> t.getType()));

         //项目日志
         Map<String, List<ProWorkLogic>>  logicMap = getProLogic(id);

         //项目成员
         String member = getMember(archives);

         //逻辑处理
             ProStatisticsRespDTO respDTO =  new ProStatisticsRespDTO(archives.getId(), archives.getProName(),
                     archives.getProCode(), archives.getProName(), "0", member);
             //配置收付款条线数据
             setSecondChildren(businesses, respDTO, CoLineMap, otherList, businessMap, logicMap, milestoneMap, purchaseDataMap, slipMap);

         return respDTO;
     }



    /**
     * @desc 获取项目组成员
     */
    private String getMember(ProjectManageArchives archives) {
        String[] member = {archives.getSaleManager(), archives.getSaleMan(), archives.getImpManager(), archives.getImpConsultant(),
        archives.getDeManager(), archives.getDeEngineer(), archives.getSeManager(), archives.getSeEngineer()};
        if(member.length == 0){
            throw new JeecgBootException("项目编码为：" + archives.getProCode() + " 的成员为空");
        }
           return String.join(",", Arrays.asList(String.join("、", Arrays.asList(member).stream().filter(t-> StringUtils.isNotBlank(t)).collect(Collectors.toList())).split("、")).stream().distinct().collect(Collectors.toList()));
    }

    /**
     * @desc 日志map
     */
    private Map<String, List<ProWorkLogic>> getProLogic(String id) {
         QueryWrapper<ProWorkLogic> queryWrapper = new QueryWrapper<>();
         queryWrapper.eq("del_flag", "0");
         queryWrapper.eq("pro_id", id);
         queryWrapper.orderByAsc("create_time");
         List<ProWorkLogic>  list = proWorkLogicService.list(queryWrapper);
         //1.开发 2.实施 3.服务
         return list.stream().collect(Collectors.groupingBy(t -> t.getType()));
    }

    /**
     * @desc 配置收付款线条数据
     */
    private void setSecondChildren(List<ProjectManageArchivesAndBusiness> businesses, ProStatisticsRespDTO respDTO, Map<String, BaseArchivesCollectionLine> CoLineMap,
        List<ProjectManageBusinessOther> otherList, Map<String, List<ProjectManageArchivesAndBusiness>> businessMap, Map<String, List<ProWorkLogic>> logicMap,
        Map<String, List<ProWorkMilestone>> milestoneMap,  Map<String, List<PurchaseDataDTO>> purchaseDataMap, Map<String, List<SlipAllListDTO>> slipMap) {
        List<ProStatisticsTwoRespDTO> list = Lists.newArrayList();

        //i： 1代表实施， 2代表开发， 3代表服务, 4,代表其他
        for(int i = 1; i <=4 ; i++) {
            if(!CollectionUtils.isEmpty(businesses)){
                List<ProjectManageArchivesAndBusiness> businesseses = selectSecondChildren(i, businesses);

                if(!CollectionUtils.isEmpty(businesseses)){
                    for(ProjectManageArchivesAndBusiness business : businesseses){
                        //只处理收款计划
                        if(business.getPlanType() == 3){
                            continue;
                        }


                        //里程碑1
                        ProStatisticsTwoRespDTO twoRespDTO = new ProStatisticsTwoRespDTO(business, respDTO.getId());
                        if(StringUtils.isNotBlank(business.getMileId1())){
                            twoRespDTO.setCoArchivesCode(CoLineMap.get(business.getPlanId()).getCode());
                            twoRespDTO.setMileId(business.getMileId1());
                            twoRespDTO.setMileName(business.getMilestone1());
                            List<ProjectManageArchivesAndBusiness> others = businessMap.get("3&"+business.getPlanId());
                            if(!CollectionUtils.isEmpty(others)){
                                twoRespDTO.setMileCoPrice(others.get(0).getPrice1());
                            }

                            twoRespDTO.setMilePrice(business.getPrice1());
                            setThreeChildren(twoRespDTO, otherList, i, logicMap, milestoneMap, purchaseDataMap, slipMap);

                        }
                        //里程碑2
                        if(StringUtils.isNotBlank(business.getMileId2())){
                            twoRespDTO.setCoArchivesCode(CoLineMap.get(business.getPlanId()).getCode());
                            twoRespDTO.setMileId(business.getMileId2());
                            twoRespDTO.setMileName(business.getMilestone2());
                            List<ProjectManageArchivesAndBusiness> others = businessMap.get("3&"+business.getPlanId());
                            if(!CollectionUtils.isEmpty(others)){
                                twoRespDTO.setMileCoPrice(others.get(0).getPrice2());
                            }
                            setThreeChildren(twoRespDTO, otherList, i, logicMap, milestoneMap, purchaseDataMap, slipMap);
                        }
                        //里程碑3
                        if(StringUtils.isNotBlank(business.getMileId3())){
                            twoRespDTO.setCoArchivesCode(CoLineMap.get(business.getPlanId()).getCode());
                            twoRespDTO.setMileId(business.getMileId3());
                            twoRespDTO.setMilePrice(business.getPrice3());
                            twoRespDTO.setMileName(business.getMilestone3());
                            List<ProjectManageArchivesAndBusiness> others = businessMap.get("3&"+business.getPlanId());
                            if(!CollectionUtils.isEmpty(others)){
                                twoRespDTO.setMileCoPrice(others.get(0).getPrice3());
                            }
                            setThreeChildren(twoRespDTO, otherList, i, logicMap, milestoneMap, purchaseDataMap, slipMap);
                        }
                        //里程碑4
                        if(StringUtils.isNotBlank(business.getMileId4())){
                            twoRespDTO.setCoArchivesCode(CoLineMap.get(business.getPlanId()).getCode());
                            twoRespDTO.setMileId(business.getMileId4());
                            twoRespDTO.setMileName(business.getMilestone4());
                            List<ProjectManageArchivesAndBusiness> others = businessMap.get("3&"+business.getPlanId());
                            if(!CollectionUtils.isEmpty(others)) {
                                twoRespDTO.setMileCoPrice(others.get(0).getPrice4());
                            }
                            setThreeChildren(twoRespDTO, otherList, i, logicMap, milestoneMap, purchaseDataMap, slipMap);
                        }
                        //里程碑5
                        if(StringUtils.isNotBlank(business.getMileId5())){
                            twoRespDTO.setCoArchivesCode(CoLineMap.get(business.getPlanId()).getCode());
                            twoRespDTO.setMileId(business.getMileId5());
                            twoRespDTO.setMilePrice(business.getPrice5());
                            twoRespDTO.setMileName(business.getMilestone5());
                            List<ProjectManageArchivesAndBusiness> others = businessMap.get("3&"+business.getPlanId());
                            if(!CollectionUtils.isEmpty(others)) {
                                twoRespDTO.setMileCoPrice(others.get(0).getPrice5());
                            }
                            setThreeChildren(twoRespDTO, otherList, i, logicMap, milestoneMap, purchaseDataMap, slipMap);
                        }

                        if(CollectionUtils.isEmpty(twoRespDTO.getChildren())){
                            twoRespDTO.setIsLeaf(true);
                            twoRespDTO.setChildren(null);
                        }
                        list.add(twoRespDTO);
                    }
                }


            } else {
              respDTO.setChildren(null);
              respDTO.setIsLeaf(true);
            }
        }
        respDTO.setChildren(list);
    }



    //配置计划信息表的数据
    private void setThreeChildren(ProStatisticsTwoRespDTO twoRespDTO, List<ProjectManageBusinessOther> otherList, int i,
         Map<String, List<ProWorkLogic>> logicMap, Map<String, List<ProWorkMilestone>> milestoneMap, Map<String, List<PurchaseDataDTO>> purchaseDataMap,
          Map<String, List<SlipAllListDTO>> slipMap) {

        List<ProStatisticsThreeRespDTO> respDTOS = Lists.newArrayList();
        //计划类型 1.商务 2.实施 3.开发 4.服务 5.其他
         i++;
         for(ProjectManageBusinessOther other : otherList){
             if(other.getPlanType() == i && other.getMuilesId().equals(twoRespDTO.getMileId())){
                 ProStatisticsThreeRespDTO threeRespDTO = new ProStatisticsThreeRespDTO(other, twoRespDTO);
                 //获取实际开始时间
                 String planType = ChoosePlanTypeEnum.match(i) != null ? ChoosePlanTypeEnum.match(i).getPlanType() : null;
                 List<ProWorkLogic> logics = logicMap.get(planType);
                 if(!CollectionUtils.isEmpty(logics)){
                     threeRespDTO.setMileRealStartDate(logics.get(0).getStartDate());
                 }

                 //配置里程碑确认单--实际结束时间
                 List<ProWorkMilestone> proWorkMilestones = milestoneMap.get(ChoosePlanTypeEnum.match(i) != null ?
                         ChoosePlanTypeEnum.match(i).getPlanType() : null);
                 if(!CollectionUtils.isEmpty(proWorkMilestones)){
                     for(ProWorkMilestone proWorkMilestone : proWorkMilestones){
                         if(proWorkMilestone.getMileId().equals(other.getMuilesId())){
                             threeRespDTO.setMileRealEndtDate(proWorkMilestone.getConfirmTime());
                         }
                     }
                 }

                 //配置 里程碑开票金额(销售发票)
                 List<PurchaseDataDTO> purchaseDataDTOS = purchaseDataMap.get("2");
                 if(!CollectionUtils.isEmpty(purchaseDataDTOS)) {
                     for(PurchaseDataDTO purchaseDataDTO : purchaseDataDTOS) {
                         if(twoRespDTO.getCoArchivesName().equals(purchaseDataDTO.getCoArchivesName()) &&
                                 threeRespDTO.getMilesName().equals(purchaseDataDTO.getProArchivesMilestone())) {
                             threeRespDTO.setMileInvoice(purchaseDataDTO.getCoPrice());
                         }
                     }
                 }

               //配置 里程碑收票(采购发票)
                 List<PurchaseDataDTO> purchaseDataDTOS2 = purchaseDataMap.get("1");
                 if(!CollectionUtils.isEmpty(purchaseDataDTOS2)) {
                     for(PurchaseDataDTO purchaseDataDTO : purchaseDataDTOS2) {
                         if(twoRespDTO.getCoArchivesName().equals(purchaseDataDTO.getCoArchivesName()) &&
                                 threeRespDTO.getMilesName().equals(purchaseDataDTO.getProArchivesMilestone())) {
                             threeRespDTO.setMileTicket(purchaseDataDTO.getCoPrice());
                         }
                     }
                 }
               //里程碑实际收款 (收款单)
                 List<SlipAllListDTO> slipAllListDTOS = slipMap.get("1");
                 if(!CollectionUtils.isEmpty(slipAllListDTOS)) {
                     for(SlipAllListDTO slipAllListDTO : slipAllListDTOS) {
                         if(slipAllListDTO.getCoArchivesName().equals(twoRespDTO.getCoArchivesName()) &&
                                 slipAllListDTO.getProArchivesMilestone().equals(threeRespDTO.getMilesName())) {
                             threeRespDTO.setMileRealCollecion(slipAllListDTO.getCoPrvice());
                         }
                     }
                 }
                 //里程碑实际收款 (收款单)
                 List<SlipAllListDTO> slipAllListDTOS2 = slipMap.get("2");
                 if(!CollectionUtils.isEmpty(slipAllListDTOS2)) {
                     for(SlipAllListDTO slipAllListDTO : slipAllListDTOS2) {
                         if(slipAllListDTO.getCoArchivesName().equals(twoRespDTO.getCoArchivesName()) &&
                                 slipAllListDTO.getProArchivesMilestone().equals(threeRespDTO.getMilesName())) {
                             threeRespDTO.setMileRealPayment(slipAllListDTO.getCoPrvice());
                         }
                     }
                 }
                 threeRespDTO.setIsLeaf(true);
                 threeRespDTO.setChilderen(null);


                 respDTOS.add(threeRespDTO);
             }
         }
          //设置其他数据
         if(i == 5){
             ProjectManageBusinessOther other = new ProjectManageBusinessOther();
             ProStatisticsThreeRespDTO threeRespDTO = new ProStatisticsThreeRespDTO(other, twoRespDTO);
             threeRespDTO.setKey(UUID.randomUUID().toString());
             threeRespDTO.setTitle(twoRespDTO.getMileName());
             threeRespDTO.setMilesName(twoRespDTO.getMileName());

             //配置 里程碑开票金额(销售发票)
             List<PurchaseDataDTO> purchaseDataDTOS = purchaseDataMap.get("2");
             if(!CollectionUtils.isEmpty(purchaseDataDTOS)) {
                 for(PurchaseDataDTO purchaseDataDTO : purchaseDataDTOS) {
                     if(twoRespDTO.getCoArchivesName().equals(purchaseDataDTO.getCoArchivesName()) &&
                             threeRespDTO.getMilesName().equals(purchaseDataDTO.getProArchivesMilestone())) {
                         threeRespDTO.setMileInvoice(purchaseDataDTO.getCoPrice());
                     }
                 }
             }

             //配置 里程碑收票(采购发票)
             List<PurchaseDataDTO> purchaseDataDTOS2 = purchaseDataMap.get("1");
             if(!CollectionUtils.isEmpty(purchaseDataDTOS2)) {
                 for(PurchaseDataDTO purchaseDataDTO : purchaseDataDTOS2) {
                     if(twoRespDTO.getCoArchivesName().equals(purchaseDataDTO.getCoArchivesName()) &&
                             threeRespDTO.getMilesName().equals(purchaseDataDTO.getProArchivesMilestone())) {
                         threeRespDTO.setMileTicket(purchaseDataDTO.getCoPrice());
                     }
                 }
             }
             //里程碑实际收款 (收款单)
             List<SlipAllListDTO> slipAllListDTOS = slipMap.get("1");
             if(!CollectionUtils.isEmpty(slipAllListDTOS)) {
                 for(SlipAllListDTO slipAllListDTO : slipAllListDTOS) {
                     if(slipAllListDTO.getCoArchivesName().equals(twoRespDTO.getCoArchivesName()) &&
                             slipAllListDTO.getProArchivesMilestone().equals(threeRespDTO.getMilesName())) {
                         threeRespDTO.setMileRealCollecion(slipAllListDTO.getCoPrvice());
                     }
                 }
             }
             //里程碑实际收款 (收款单)
             List<SlipAllListDTO> slipAllListDTOS2 = slipMap.get("2");
             if(!CollectionUtils.isEmpty(slipAllListDTOS2)) {
                 for(SlipAllListDTO slipAllListDTO : slipAllListDTOS2) {
                     if(slipAllListDTO.getCoArchivesName().equals(twoRespDTO.getCoArchivesName()) &&
                             slipAllListDTO.getProArchivesMilestone().equals(threeRespDTO.getMilesName())) {
                         threeRespDTO.setMileRealPayment(slipAllListDTO.getCoPrvice());
                     }
                 }
             }
             threeRespDTO.setIsLeaf(true);
             threeRespDTO.setChilderen(null);
             respDTOS.add(threeRespDTO);

         }


         if(CollectionUtils.isEmpty(twoRespDTO.getChildren())){
             twoRespDTO.setChildren(CollectionUtils.isEmpty(respDTOS)? null : respDTOS);
             return;
         }
        twoRespDTO.setChildren(CollectionUtils.isEmpty(respDTOS)? null : Stream.concat(twoRespDTO.getChildren().stream(),
                respDTOS.stream()).collect(Collectors.toList()));
    }



    //根据首付款条线获取指定的数据
    private List<ProjectManageArchivesAndBusiness> selectSecondChildren(int i, List<ProjectManageArchivesAndBusiness> businesses) {
        switch(i){
            //代表实施
            case 1:
                return businesses.stream().filter(t -> t.getPlanName().indexOf("实施") != -1).collect(Collectors.toList());
            case 2:
                return businesses.stream().filter(t -> t.getPlanName().indexOf("开发") != -1).collect(Collectors.toList());
            case 3:
                return businesses.stream().filter(t -> t.getPlanName().indexOf("服务") != -1).collect(Collectors.toList());
            case 4:
                return businesses.stream().filter(t -> t.getPlanName().indexOf("实施") == -1 && t.getPlanName().indexOf("开发") == -1
                && t.getPlanName().indexOf("服务") == -1).collect(Collectors.toList());
        }

           return null;
    }

    /**
      * @desc 获取项目档案里程碑信息表数据
      */
     private List<ProjectManageArchivesAndBusiness> getBusinessData(String id) {
         QueryWrapper<ProjectManageArchivesAndBusiness> queryWrapper = new QueryWrapper<>();
         Integer[] planTypeArr = {1, 3};
         queryWrapper.eq("del_flag", "0").eq("pro_archives_id", id).in("plan_type", planTypeArr);
         queryWrapper.orderByAsc("plan_type");
         List<ProjectManageArchivesAndBusiness> list = projectManageArchivesAndBusinessService.list(queryWrapper);
         return list;
     }
 }

