package org.jeecg.modules.system.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.entity.SysAnnouncementSend;
import org.jeecg.modules.system.entity.SysUser;
import org.jeecg.modules.system.service.ISysAnnouncementSendService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 用户通告阅读标记表
 * @author： jeecg-boot
 * @date： 2019-11-22
 * @version： V1.0
 */
@RestController
@RequestMapping("/sys/sysAnnouncementSend")
@Slf4j
public class SysAnnouncementSendController {
    @Autowired
    private ISysAnnouncementSendService sysAnnouncementSendService;

    /**
     * 分页列表查询（有权限）
     *
     * @param sysAnnouncementSend
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @RequiresPermissions("url:message:view")
    public Result<IPage<SysAnnouncementSend>> queryPageList(SysAnnouncementSend sysAnnouncementSend,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<SysAnnouncementSend>> result = new Result<IPage<SysAnnouncementSend>>();

        //阅读状态
        String readFlag = sysAnnouncementSend.getReadFlag();
        //标题
        String title = sysAnnouncementSend.getTitile();

        if (StringUtils.isNotBlank(readFlag)) {
            sysAnnouncementSend.setReadFlag(null);
        }
        if (StringUtils.isNotBlank(title)) {
            sysAnnouncementSend.setTitile(null);
        }

        QueryWrapper<SysAnnouncementSend> queryWrapper = QueryGenerator.initQueryWrapper(sysAnnouncementSend, req.getParameterMap());
        Page<SysAnnouncementSend> page = new Page<SysAnnouncementSend>(pageNo, pageSize);
        // IPage<SysAnnouncementSend> pageList = sysAnnouncementSendService.page(page, queryWrapper);
        SysUser user = (SysUser) SecurityUtils.getSubject().getPrincipal();
        queryWrapper.eq("sas.user_id", user.getPersonnelId());
        if (StringUtils.isNotBlank(readFlag)) {
            queryWrapper.eq("sas.read_flag", readFlag);
        }
        if (StringUtils.isNotBlank(title)) {
            queryWrapper.like("sa.titile", title);
        }
        queryWrapper.orderByDesc("sa.create_time");
        IPage<SysAnnouncementSend> pageList = sysAnnouncementSendService.getPageList(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询(无权限)
     *
     * @param sysAnnouncementSend
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/listNotJurisdiction")
    public Result<IPage<SysAnnouncementSend>> listNotJurisdiction(SysAnnouncementSend sysAnnouncementSend,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<SysAnnouncementSend>> result = new Result<IPage<SysAnnouncementSend>>();

        //阅读状态
        String readFlag = sysAnnouncementSend.getReadFlag();
        //标题
        String title = sysAnnouncementSend.getTitile();

        if (StringUtils.isNotBlank(readFlag)) {
            sysAnnouncementSend.setReadFlag(null);
        }
        if (StringUtils.isNotBlank(title)) {
            sysAnnouncementSend.setTitile(null);
        }

        QueryWrapper<SysAnnouncementSend> queryWrapper = QueryGenerator.initQueryWrapper(sysAnnouncementSend, req.getParameterMap());
        Page<SysAnnouncementSend> page = new Page<SysAnnouncementSend>(pageNo, pageSize);
        // IPage<SysAnnouncementSend> pageList = sysAnnouncementSendService.page(page, queryWrapper);
        SysUser user = (SysUser) SecurityUtils.getSubject().getPrincipal();
        queryWrapper.eq("sas.user_id", user.getPersonnelId());
        if (StringUtils.isNotBlank(readFlag)) {
            queryWrapper.eq("sas.read_flag", readFlag);
        }
        if (StringUtils.isNotBlank(title)) {
            queryWrapper.like("sa.titile", title);
        }
        IPage<SysAnnouncementSend> pageList = sysAnnouncementSendService.getPageList(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param sysAnnouncementSend
     * @return
     */
    @PostMapping(value = "/add")
    public Result<SysAnnouncementSend> add(@RequestBody SysAnnouncementSend sysAnnouncementSend) {
        Result<SysAnnouncementSend> result = new Result<SysAnnouncementSend>();
        try {
            sysAnnouncementSendService.save(sysAnnouncementSend);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param sysAnnouncementSend
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<SysAnnouncementSend> edit(@RequestBody SysAnnouncementSend sysAnnouncementSend) {
        Result<SysAnnouncementSend> result = new Result<SysAnnouncementSend>();
        SysAnnouncementSend sysAnnouncementSendEntity = sysAnnouncementSendService.getById(sysAnnouncementSend.getId());
        if (sysAnnouncementSendEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = sysAnnouncementSendService.updateById(sysAnnouncementSend);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<SysAnnouncementSend> delete(@RequestParam(name = "id", required = true) String id) {
        Result<SysAnnouncementSend> result = new Result<SysAnnouncementSend>();
        SysAnnouncementSend sysAnnouncementSend = sysAnnouncementSendService.getById(id);
        if (sysAnnouncementSend == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = sysAnnouncementSendService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<SysAnnouncementSend> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<SysAnnouncementSend> result = new Result<SysAnnouncementSend>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.sysAnnouncementSendService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<SysAnnouncementSend> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<SysAnnouncementSend> result = new Result<SysAnnouncementSend>();
        SysAnnouncementSend sysAnnouncementSend = sysAnnouncementSendService.getById(id);
        if (sysAnnouncementSend == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(sysAnnouncementSend);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<SysAnnouncementSend> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                SysAnnouncementSend sysAnnouncementSend = JSON.parseObject(deString, SysAnnouncementSend.class);
                queryWrapper = QueryGenerator.initQueryWrapper(sysAnnouncementSend, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<SysAnnouncementSend> pageList = sysAnnouncementSendService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "用户通告阅读标记表列表");
        mv.addObject(NormalExcelConstants.CLASS, SysAnnouncementSend.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("用户通告阅读标记表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<SysAnnouncementSend> listSysAnnouncementSends = ExcelImportUtil.importExcel(file.getInputStream(), SysAnnouncementSend.class, params);
                for (SysAnnouncementSend sysAnnouncementSendExcel : listSysAnnouncementSends) {
                    sysAnnouncementSendService.save(sysAnnouncementSendExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listSysAnnouncementSends.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }


    /**
     * 通过id查询系统通告表相关信息
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryAnnouncementById")
    public Result<SysAnnouncementSend> queryAnnouncementById(@RequestParam(name = "id", required = true) String id) {
        Result<SysAnnouncementSend> result = new Result<>();
        SysAnnouncementSend sysAnnouncementSend = sysAnnouncementSendService.queryAnnouncementById(id);
        if (sysAnnouncementSend == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(sysAnnouncementSend);
            result.setSuccess(true);
        }
        return result;
    }

    /***
     * @Title:修改阅读状态
            * @Description:TODO
            * @param id
            * @return org.jeecg.common.api.vo.Result
            * @exception
            * @author 陈创
            * @date 2019/11/27 11:19
            * @updatehistory:
     *   2019/11/27 11:19 修改
            */
    @GetMapping(value = "/updateReadFlagById")
    public Result updateReadFlagById(@RequestParam(name = "id", required = true) String id) {
        Result result = new Result();
        SysAnnouncementSend sysAnnouncementSend = sysAnnouncementSendService.queryAnnouncementById(id);
        if (sysAnnouncementSend == null) {
            result.error500("未找到对应实体");
        } else {
            try {
                sysAnnouncementSend.setReadFlag("1");
                sysAnnouncementSendService.updateById(sysAnnouncementSend);
                result.setSuccess(true);
            } catch (Exception e) {
                e.printStackTrace();
                result.error500("操作失败："+e.getMessage());
            }
        }
        return result;
    }


}
