package org.jeecg.modules.system.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.entity.SysSerialPattern;
import org.jeecg.modules.system.service.ISysSerialPatternService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.system.vo.CallResult;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 编号模式定义表
 * @author： jeecg-boot
 * @date：   2019-05-23
 * @version： V1.0
 */
@RestController
@RequestMapping("/sys/serialPattern")
@Slf4j
public class SysSerialPatternController {
	@Autowired
	private ISysSerialPatternService sysSerialPatternService;
	
	/**
	  * 分页列表查询
	 * @param sysSerialPattern
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<SysSerialPattern>> queryPageList(SysSerialPattern sysSerialPattern,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<SysSerialPattern>> result = new Result<IPage<SysSerialPattern>>();
		//表名
		String tableName = sysSerialPattern.getTableName();
		if (StringUtils.isNotBlank(sysSerialPattern.getTableName())) {
			sysSerialPattern.setTableName(null);
		}
		//字段名
		String fieldName = sysSerialPattern.getFieldName();
		if (StringUtils.isNotBlank(sysSerialPattern.getFieldName())) {
			sysSerialPattern.setFieldName(null);
		}
		//编号模式
		String serialPattern = sysSerialPattern.getSerialPattern();
		if (StringUtils.isNotBlank(sysSerialPattern.getSerialPattern())) {
			sysSerialPattern.setSerialPattern(null);
		}
		QueryWrapper<SysSerialPattern> queryWrapper = QueryGenerator.initQueryWrapper(sysSerialPattern, req.getParameterMap());
		if (StringUtils.isNotBlank(tableName)) {
			queryWrapper.like("table_name", tableName);
		}
		if (StringUtils.isNotBlank(fieldName)) {
			queryWrapper.like("field_name", fieldName);
		}
		if (StringUtils.isNotBlank(serialPattern)) {
			queryWrapper.like("serial_pattern", serialPattern);
		}
		Page<SysSerialPattern> page = new Page<SysSerialPattern>(pageNo, pageSize);
		IPage<SysSerialPattern> pageList = sysSerialPatternService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}


	 /**
	  * 分页列表查询（有权限）
	  * @param sysSerialPattern
	  * @param pageNo
	  * @param pageSize
	  * @param req
	  * @return
	  */
	 @GetMapping(value = "/list1")
	 @RequiresPermissions("url:serial-pattern:view")
	 public Result<IPage<SysSerialPattern>> queryPageList1(SysSerialPattern sysSerialPattern,
														  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														  HttpServletRequest req) {
		 Result<IPage<SysSerialPattern>> result = new Result<IPage<SysSerialPattern>>();
		 //表名
		 String tableName = sysSerialPattern.getTableName();
		 if (StringUtils.isNotBlank(sysSerialPattern.getTableName())) {
			 sysSerialPattern.setTableName(null);
		 }
		 //字段名
		 String fieldName = sysSerialPattern.getFieldName();
		 if (StringUtils.isNotBlank(sysSerialPattern.getFieldName())) {
			 sysSerialPattern.setFieldName(null);
		 }
		 //编号模式
		 String serialPattern = sysSerialPattern.getSerialPattern();
		 if (StringUtils.isNotBlank(sysSerialPattern.getSerialPattern())) {
			 sysSerialPattern.setSerialPattern(null);
		 }
		 QueryWrapper<SysSerialPattern> queryWrapper = QueryGenerator.initQueryWrapper(sysSerialPattern, req.getParameterMap());
		 if (StringUtils.isNotBlank(tableName)) {
			 queryWrapper.like("table_name", tableName);
		 }
		 if (StringUtils.isNotBlank(fieldName)) {
			 queryWrapper.like("field_name", fieldName);
		 }
		 if (StringUtils.isNotBlank(serialPattern)) {
			 queryWrapper.like("serial_pattern", serialPattern);
		 }
		 Page<SysSerialPattern> page = new Page<SysSerialPattern>(pageNo, pageSize);
		 IPage<SysSerialPattern> pageList = sysSerialPatternService.page(page, queryWrapper);
		 result.setSuccess(true);
		 result.setResult(pageList);
		 return result;
	 }
	
	/**
	  *   添加
	 * @param sysSerialPattern
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<SysSerialPattern> add(@RequestBody @Valid SysSerialPattern sysSerialPattern, BindingResult bindingResult) {
		Result<SysSerialPattern> result = new Result<SysSerialPattern>();
		try {

			CallResult<String> callResult = sysSerialPatternService.isSerialPatternValid(sysSerialPattern.getSerialPattern());
			if (!callResult.isSucceed()){
				result.error500(callResult.GetFullMessage());
				return result;
			}

			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			sysSerialPatternService.save(sysSerialPattern);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param sysSerialPattern
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	public Result<SysSerialPattern> edit(@RequestBody @Valid SysSerialPattern sysSerialPattern, BindingResult bindingResult) {
		Result<SysSerialPattern> result = new Result<SysSerialPattern>();
		SysSerialPattern sysSerialPatternEntity = sysSerialPatternService.getById(sysSerialPattern.getId());

		CallResult<String> callResult = sysSerialPatternService.isSerialPatternValid(sysSerialPattern.getSerialPattern());
		if (!callResult.isSucceed()){
			result.error500(callResult.GetFullMessage());
			return result;
		}
		if(sysSerialPatternEntity==null) {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			result.error500("未找到对应实体");
		}else {
			boolean ok = sysSerialPatternService.updateById(sysSerialPattern);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<SysSerialPattern> delete(@RequestParam(name="id",required=true) String id) {
		Result<SysSerialPattern> result = new Result<SysSerialPattern>();
		SysSerialPattern sysSerialPattern = sysSerialPatternService.getById(id);
		if(sysSerialPattern==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = sysSerialPatternService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<SysSerialPattern> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<SysSerialPattern> result = new Result<SysSerialPattern>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.sysSerialPatternService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<SysSerialPattern> queryById(@RequestParam(name="id",required=true) String id) {
		Result<SysSerialPattern> result = new Result<SysSerialPattern>();
		SysSerialPattern sysSerialPattern = sysSerialPatternService.getById(id);
		if(sysSerialPattern==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(sysSerialPattern);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<SysSerialPattern> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              SysSerialPattern sysSerialPattern = JSON.parseObject(deString, SysSerialPattern.class);
              queryWrapper = QueryGenerator.initQueryWrapper(sysSerialPattern, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<SysSerialPattern> pageList = sysSerialPatternService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "编号模式定义表列表");
      mv.addObject(NormalExcelConstants.CLASS, SysSerialPattern.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("编号模式定义表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<SysSerialPattern> listSysSerialPatterns = ExcelImportUtil.importExcel(file.getInputStream(), SysSerialPattern.class, params);
              for (SysSerialPattern sysSerialPatternExcel : listSysSerialPatterns) {
                  sysSerialPatternService.save(sysSerialPatternExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listSysSerialPatterns.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
